/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.solr.handler.admin.api;

import static org.apache.solr.client.solrj.SolrRequest.METHOD.GET;
import static org.apache.solr.security.PermissionNameProvider.Name.HEALTH_PERM;

import org.apache.solr.api.EndPoint;
import org.apache.solr.handler.admin.HealthCheckHandler;
import org.apache.solr.request.SolrQueryRequest;
import org.apache.solr.response.SolrQueryResponse;

/**
 * V2 API for checking the health of the receiving node.
 *
 * <p>This API (GET /v2/node/health) is analogous to the v1 /admin/info/health.
 */
public class NodeHealthAPI {
  private final HealthCheckHandler handler;

  public NodeHealthAPI(HealthCheckHandler handler) {
    this.handler = handler;
  }

  // TODO Update permission here once SOLR-11623 lands.
  @EndPoint(
      path = {"/node/health"},
      method = GET,
      permission = HEALTH_PERM)
  public void getSystemInformation(SolrQueryRequest req, SolrQueryResponse rsp) throws Exception {
    handler.handleRequestBody(req, rsp);
  }
}
