/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.solr.analytics.function.field;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import org.apache.solr.analytics.ExpressionFactory;
import org.junit.Test;

public class StringFieldsTest extends AbstractAnalyticsFieldTest {

  @Test
  public void expressionFactoryCreationTest() {
    ExpressionFactory fact = getExpressionFactory();

    assertTrue(fact.createExpression("string_s") instanceof StringField);
    assertTrue(fact.createExpression("string_sm") instanceof StringMultiField);
  }

  @Test
  public void singleValuedStringTest() throws IOException {
    StringField valueField = new StringField("string_s");
    Map<String, String> values = new HashMap<>();

    Set<String> missing =
        collectFieldValues(
            valueField,
            id -> {
              String value = valueField.getString();
              if (valueField.exists()) {
                values.put(id, value);
              }
              return valueField.exists();
            });

    checkSingleFieldValues(singleStrings, values, missing);
  }

  @Test
  public void multiValuedStringTest() throws IOException {
    StringMultiField valueField = new StringMultiField("string_sm");
    Map<String, Map<String, Integer>> values = new HashMap<>();

    Set<String> missing =
        collectFieldValues(
            valueField,
            id -> {
              Map<String, Integer> doc = new HashMap<>();
              valueField.streamStrings(
                  value -> {
                    doc.put(value, doc.getOrDefault(value, 0) + 1);
                  });
              if (doc.size() > 0) {
                values.put(id, doc);
              }
              return doc.size() > 0;
            });

    checkMultiFieldValues(multiStrings, values, missing, true);
  }
}
