/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.solr.client.solrj.io.eval;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import org.apache.commons.math3.util.MathArrays;
import org.apache.solr.client.solrj.io.stream.expr.StreamExpression;
import org.apache.solr.client.solrj.io.stream.expr.StreamFactory;

public class SequenceEvaluator extends RecursiveNumericEvaluator implements ManyValueWorker {
  protected static final long serialVersionUID = 1L;

  public SequenceEvaluator(StreamExpression expression, StreamFactory factory) throws IOException {
    super(expression, factory);

    if (3 != containedEvaluators.size()) {
      throw new IOException(
          String.format(
              Locale.ROOT,
              "Invalid expression %s - expecting three values but found %d",
              expression,
              containedEvaluators.size()));
    }
  }

  @Override
  public Object doWork(Object... values) throws IOException {
    if (3 != values.length) {
      throw new IOException(
          String.format(
              Locale.ROOT,
              "%s(...) only works with 3 values but %d were provided",
              constructingFactory.getFunctionName(getClass()),
              values.length));
    }

    Number sizeNum = (Number) values[0];
    Number startNum = (Number) values[1];
    Number strideNum = (Number) values[2];

    List<Number> sequence = new ArrayList<>(sizeNum.intValue());
    for (int number :
        MathArrays.sequence(sizeNum.intValue(), startNum.intValue(), strideNum.intValue())) {
      sequence.add(number);
    }

    return sequence;
  }
}
